From 08f12f1eff7ad40003b2fa907b32bd407695c495 Mon Sep 17 00:00:00 2001
From: Peter Smith <peter.b.smith@fujitsu.com>
Date: Thu, 4 Feb 2021 18:45:32 +1100
Subject: [PATCH v1] replorigin_drop_by_name.

Drop the origin by name as a single operation.
The whole function is locked to prevent concurrent drops.

Discussion: https://www.postgresql.org/message-id/CAHut%2BPuW8DWV5fskkMWWMqzt-x7RPcNQOtJQBp6SdwyRghCk7A%40mail.gmail.com
---
 src/backend/replication/logical/origin.c | 24 +++++++++++++++++++-----
 src/include/replication/origin.h         |  1 +
 2 files changed, 20 insertions(+), 5 deletions(-)

diff --git a/src/backend/replication/logical/origin.c b/src/backend/replication/logical/origin.c
index 9bd761a..00b182d 100644
--- a/src/backend/replication/logical/origin.c
+++ b/src/backend/replication/logical/origin.c
@@ -1250,22 +1250,36 @@ pg_replication_origin_create(PG_FUNCTION_ARGS)
 }
 
 /*
+ * Drop a named origin. Lock entire function to prevent races.
+ */
+void
+replorigin_drop_by_name(char *name, bool missing_ok, bool nowait)
+{
+	RepOriginId roident;
+
+	/* lock to prevent the replication origin from vanishing */
+	LockRelationOid(ReplicationOriginRelationId, RowExclusiveLock);
+	
+	roident = replorigin_by_name(name, missing_ok);
+	if (OidIsValid(roident))
+		replorigin_drop(roident, nowait);
+
+	UnlockRelationOid(ReplicationOriginRelationId, RowExclusiveLock);
+}
+
+/*
  * Drop replication origin.
  */
 Datum
 pg_replication_origin_drop(PG_FUNCTION_ARGS)
 {
 	char	   *name;
-	RepOriginId roident;
 
 	replorigin_check_prerequisites(false, false);
 
 	name = text_to_cstring((text *) DatumGetPointer(PG_GETARG_DATUM(0)));
 
-	roident = replorigin_by_name(name, false);
-	Assert(OidIsValid(roident));
-
-	replorigin_drop(roident, true);
+	replorigin_drop_by_name(name, false /*missing_ok */, true /* nowait */);	
 
 	pfree(name);
 
diff --git a/src/include/replication/origin.h b/src/include/replication/origin.h
index 731445a..e13c238 100644
--- a/src/include/replication/origin.h
+++ b/src/include/replication/origin.h
@@ -41,6 +41,7 @@ extern PGDLLIMPORT TimestampTz replorigin_session_origin_timestamp;
 extern RepOriginId replorigin_by_name(char *name, bool missing_ok);
 extern RepOriginId replorigin_create(char *name);
 extern void replorigin_drop(RepOriginId roident, bool nowait);
+extern void replorigin_drop_by_name(char *name, bool missing_ok, bool nowait);
 extern bool replorigin_by_oid(RepOriginId roident, bool missing_ok,
 							  char **roname);
 
-- 
1.8.3.1

